/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef NULL_DATA_IO
#define NULL_DATA_IO

#include "dataio/PortableDataIO.h"

/**
 *  Data I/O equivalent to /dev/null.  
 */
class NullDataIO : public PortableDataIO
{
public:
   /** Default Constructor. */
   NullDataIO() : _shutdown(false) {/* empty */}

   /** Virtual Destructor, to keep C++ honest */
   virtual ~NullDataIO() {/* empty */}

   /** 
    *  No-op method, always returns zero (except if Shutdown() was called).
    *  @param buffer Points to a buffer to read bytes into (ignored).
    *  @param size Number of bytes in the buffer (ignored).
    *  @return zero.
    */
   virtual ssize_t Read(void * /*buffer*/, size_t /*size*/)  {return _shutdown ? -1 : 0;}

   /** 
    *  No-op method, always returns (size) (except if Shutdown() was called).
    *  @param buffer Points to a buffer to write bytes from (ignored).
    *  @param size Number of bytes in the buffer (ignored).
    *  @return (size).
    */
   virtual ssize_t Write(const void * /*buffer*/, size_t size) {return _shutdown ? -1 : size;}

   /**
    *  This method always returns B_ERROR.
    */
   virtual status_t Seek(int64 /*seekOffset*/, int /*whence*/) {return B_ERROR;}

   /** 
    *  No-op method.
    *  This method doesn't do anything at all.
    */
   virtual void FlushOutput() {/* empty */}

   /** Disable us! */ 
   virtual void Shutdown() {_shutdown = true;}

private
   bool _shutdown;
};

#endif
